/* See license.txt for terms of usage */

.panelNode-html {
    -moz-box-sizing: padding-box;
    padding: 4px 0 0 2px;
}

.nodeBox {
    position: relative;
    font-family: Monaco, monospace;
    padding-left: 13px;
    -moz-user-select: -moz-none;
}
.nodeBox.search-selection {
    -moz-user-select: text;
}
.twisty {
    position: absolute;
    left: 0px;
    top: 0px;
    width: 14px;
    height: 14px;
}

.nodeChildBox {
    margin-left: 12px;
    display: none;
}

.nodeLabel,
.nodeCloseLabel {
    margin: -2px 2px 0 2px;
    border: 2px solid transparent;
    -moz-border-radius: 3px;
    padding: 0 2px;
    color: #000088;
}

.nodeCloseLabel {
    display: none;
}

.nodeTag {
    cursor: pointer;
    color: blue;
}

.nodeValue {
    color: #FF0000;
    font-weight: normal;
}

.nodeText,
.nodeComment {
    margin: 0 2px;
    vertical-align: top;
}

.nodeText {
    color: #333333;
}

.nodeWhiteSpace {
    border: 1px solid LightGray;
    white-space: pre; /* otherwise the border will be collapsed around zero pixels */
    margin-left: 1px;
    color: gray;
}


.nodeWhiteSpace_Space {
    border: 1px solid #ddd;
}

.nodeTextEntity {
    border: 1px solid gray;
    white-space: pre; /* otherwise the border will be collapsed around zero pixels */
    margin-left: 1px;
}

.nodeComment {
    color: DarkGreen;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

.nodeBox.highlightOpen > .nodeLabel {
    background-color: #EEEEEE;
}

.nodeBox.highlightOpen > .nodeCloseLabel,
.nodeBox.highlightOpen > .nodeChildBox,
.nodeBox.open > .nodeCloseLabel,
.nodeBox.open > .nodeChildBox {
    display: block;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

.nodeBox.selected > .nodeLabel > .nodeLabelBox,
.nodeBox.selected > .nodeLabel {
    border-color: Highlight;
    background-color: Highlight;
    color: HighlightText !important;
}

.nodeBox.selected > .nodeLabel > .nodeLabelBox,
.nodeBox.selected > .nodeLabel > .nodeLabelBox > .nodeTag,
.nodeBox.selected > .nodeLabel > .nodeLabelBox > .nodeAttr > .nodeValue,
.nodeBox.selected > .nodeLabel > .nodeLabelBox > .nodeText {
    color: inherit !important;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

.nodeBox.highlighted > .nodeLabel {
    border-color: Highlight !important;
    background-color: cyan !important;
    color: #000000 !important;
}

.nodeBox.highlighted > .nodeLabel > .nodeLabelBox,
.nodeBox.highlighted > .nodeLabel > .nodeLabelBox > .nodeTag,
.nodeBox.highlighted > .nodeLabel > .nodeLabelBox > .nodeAttr > .nodeValue,
.nodeBox.highlighted > .nodeLabel > .nodeLabelBox > .nodeText {
    color: #000000 !important;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

.nodeBox.nodeHidden .nodeLabel > .nodeLabelBox,
.nodeBox.nodeHidden .nodeCloseLabel,
.nodeBox.nodeHidden .nodeLabel > .nodeLabelBox > .nodeText,
.nodeBox.nodeHidden .nodeText {
    color: #888888;
}

.nodeBox.nodeHidden .nodeLabel > .nodeLabelBox > .nodeTag,
.nodeBox.nodeHidden .nodeCloseLabel > .nodeCloseLabelBox > .nodeTag {
    color: #5F82D9;
}

.nodeBox.nodeHidden .nodeLabel > .nodeLabelBox > .nodeAttr > .nodeValue {
    color: #D86060;
}

.nodeBox.nodeHidden.selected > .nodeLabel > .nodeLabelBox,
.nodeBox.nodeHidden.selected > .nodeLabel > .nodeLabelBox > .nodeTag,
.nodeBox.nodeHidden.selected > .nodeLabel > .nodeLabelBox > .nodeAttr > .nodeValue,
.nodeBox.nodeHidden.selected > .nodeLabel > .nodeLabelBox > .nodeText {
    color: SkyBlue !important;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

.nodeBox.mutated > .nodeLabel,
.nodeAttr.mutated,
.nodeValue.mutated,
.nodeText.mutated,
.nodeBox.mutated > .nodeText {
    background-color: #EFFF79;
    color: #FF0000 !important;
}

.nodeBox.selected.mutated > .nodeLabel,
.nodeBox.selected.mutated > .nodeLabel > .nodeLabelBox,
.nodeBox.selected > .nodeLabel > .nodeLabelBox > .nodeAttr.mutated > .nodeValue,
.nodeBox.selected > .nodeLabel > .nodeLabelBox > .nodeAttr > .nodeValue.mutated,
.nodeBox.selected > .nodeLabel > .nodeLabelBox > .nodeText.mutated {
    background-color: #EFFF79;
    border-color: #EFFF79;
    color: #FF0000 !important;
}

/************************************************************************************************/

.logRow-dirxml {
    padding-left: 0;
}

.soloElement > .nodeBox  {
    padding-left: 0;
}

.useA11y .nodeLabel.focused {
    outline: 2px solid #FF9933;
    -moz-outline-radius: 3px;
    outline-offset: -2px;
}

.useA11y .nodeLabelBox:focus {
    outline: none;
}

/************************************************************************************************/

.breakpointCode .twisty {
    display: none;
}

.breakpointCode .nodeBox.containerNodeBox,
.breakpointCode .nodeLabel {
    padding-left: 0px;
    margin-left: 0px;
    font-family: Monaco, monospace !important;
}

.breakpointCode .nodeTag,
.breakpointCode .nodeAttr,
.breakpointCode .nodeText,
.breakpointCode .nodeValue,
.breakpointCode .nodeLabel {
    color: DarkGreen !important;
}

.breakpointMutationType {
    position: absolute;
    top: 4px;
    right: 20px;
    color: gray;
}






/************************************************************************************************/
/************************************************************************************************/
/************************************************************************************************/
/************************************************************************************************/
/************************************************************************************************/
/************************************************************************************************/
/************************************************************************************************/
/************************************************************************************************/
/************************************************************************************************/
/************************************************************************************************/



/************************************************************************************************/
/* Twisties */

.twisty,
.logRow-errorMessage > .hasTwisty > .errorTitle,
.logRow-log > .objectBox-array.hasTwisty,
.logRow-spy .spyHead .spyTitle,
.logGroup > .logRow,
.memberRow.hasChildren > .memberLabelCell > .memberLabel,
.hasHeaders .netHrefLabel,
.netPageRow > .netCol > .netPageTitle {
    background-image: url(twistyClosed.png);
    background-repeat: no-repeat;
    background-position: 2px 2px;
	min-height: 12px;
}

.logRow-errorMessage > .hasTwisty.opened > .errorTitle,
.logRow-log > .objectBox-array.hasTwisty.opened,
.logRow-spy.opened .spyHead .spyTitle,
.logGroup.opened > .logRow,
.memberRow.hasChildren.opened > .memberLabelCell > .memberLabel,
.nodeBox.highlightOpen > .nodeLabel > .twisty,
.nodeBox.open > .nodeLabel > .twisty,
.netRow.opened > .netCol > .netHrefLabel,
.netPageRow.opened > .netCol > .netPageTitle {
    background-image: url(twistyOpen.png);
}

.twisty {
    background-position: 4px 4px;
}