/**
 * Copyright (c) Jupyter Development Team.
 * Distributed under the terms of the Modified BSD License.
 */

/* hide extraneous cell parts when in dashboard view  */
.jupyter-dashboard .celltoolbar,
.jupyter-dashboard .container .cell .prompt,
.jupyter-dashboard .container .cell .output_stderr,
.jupyter-dashboard .container .cell .output_error {
    display: none !important; /* Necessary to override inline style set by notebook */
}

/* Dashboard cell */
.jupyter-dashboard .cell {
    position: relative;
}
.jupyter-dashboard .cell,
.jupyter-dashboard .cell.selected {
    background: transparent;
    border: none;
    padding: 0;
}
.jupyter-dashboard .cell.selected:before {
    background: transparent;
}

/* Show cell code in the background of each cell */
.jupyter-dashboard:not(.view-only) .cell > .input {
    position: absolute;
    top: 0;
    left: 0;
    height: 100%;
    padding: 0.5em 1em;
    overflow: hidden;
    opacity: 0;
    transition: opacity 0.3s;
}
.jupyter-dashboard:not(.view-only) #notebook-container:not(.hide-code):hover .cell > .input {
    opacity: 0.3;
}
.jupyter-dashboard:not(.view-only) #notebook-container:not(.hide-code) .cell > .input div.input_area {
    border: none;
    background: none;
}

/* Dashboard cell controls */
.jupyter-dashboard .cell .cell-control-container {
    display: flex;
    position: absolute;
    color: #b2b2b2;
    visibility: hidden;

    background-color: #fff;
    border: 1px solid #eee;

    /* widgets overlap, so pull forward */
    z-index: 100;
}
.jupyter-dashboard:not(.view-only) .cell:hover .cell-control-container {
    visibility: visible;
}
.jupyter-dashboard .cell .cell-control-container.cell-control-nw {
    border-top: none;
    border-left: none;
    top: 0;
    left: 0;
}
.jupyter-dashboard .cell .cell-control-container.cell-control-ne {
    border-top: none;
    border-right: none;
    top: 0;
    right: 0;
}
.jupyter-dashboard .cell .cell-control-container.cell-control-se {
    border-bottom: none;
    border-right: none;
    bottom: 0;
    right: 0;
}
.jupyter-dashboard .cell .cell-control {
    padding: 5px;
    width: 25px;
    height: 25px;
    text-align: center;
    cursor: pointer;
}
.jupyter-dashboard .cell .cell-control:hover {
    color: #306c9e;
}
.jupyter-dashboard .cell .cell-control.hide-btn:hover {
    color: red;
}

/* Hidden Cells */
.jupyter-dashboard #dashboard-hidden-header {
    height: 150px;
    padding: 15px;
}
.jupyter-dashboard #dashboard-hidden-header .header {
    display: -webkit-flex;
    display: -ms-flexbox;
    display: flex;
    -webkit-align-items: center;
        -ms-flex-align: center;
            align-items: center;
    margin: 2em 0;
    padding-top: 50px;
}
.jupyter-dashboard #dashboard-hidden-header .header .title {
    margin: 0 1em 0 0;
}
.jupyter-dashboard #dashboard-hidden-header .header .btn:not(.btn-info) {
    background-color: transparent;
    border: 1px solid black;
}

/* Dashboard preview */
.jupyter-dashboard.view-only #dashboard-hidden-header {
    display: none;
}
.jupyter-dashboard.view-only .cell > .input {
    display: none !important; /* Necessary to override inline style set by notebook */
}

/* Edit cell animation */
.cell.edit-select {
    -webkit-animation: shadow-fade 2s linear 250ms;
            animation: shadow-fade 2s linear 250ms;
}
@-webkit-keyframes shadow-fade {
    0%   { box-shadow: 0 0 10px 2px #5bc0de; }
    100% { box-shadow: 0 0 10px 2px transparent; }
}
@keyframes shadow-fade {
    0%   { box-shadow: 0 0 10px 2px #5bc0de; }
    100% { box-shadow: 0 0 10px 2px transparent; }
}

/* Declarative widgets specific styles */
.jupyter-dashboard .cell .urth-widget-error {
    display: none;
}
